/*
 * Photo Image Print System
 * Copyright (C) 2001-2005 EPSON AVASYS Corporation.
 * Copyright (C) Seiko Epson Corporation 2001-2009.
 *
 * This file is part of the `ekpd' program.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, AVASYS Corporation gives permission
 * to link the code of this program with the `cbt' library and
 * distribute linked combinations including the two.  You must obey
 * the GNU General Public License in all respects for all of the
 * code used other then `cbt'.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <dirent.h>
#include "cbtd.h"


#define DEVICEID_LENGTH 512
#define PATH_SIZE 256

/* initialize system */

#define DEVICEID_LENGTH 512
static int /* Return 1 when the searching printer is found or 0. */
is_escp_printer(const char * usblp)
{
	char deviceid [DEVICEID_LENGTH];
	char usbpath [PATH_MAX];
	memset(usbpath, 0x00, sizeof(usbpath));
	snprintf(usbpath, sizeof(usbpath), "%s/%s", "/dev/usb", usblp);

	int result = 0; /* not found */
	int fd = open(usbpath, O_RDWR);

	if (fd >= 0) {
		memset(deviceid, 0x00, DEVICEID_LENGTH);

		/* GET_DEVICE_ID = 1 */
		int iostatus = ioctl(fd, _IOC(_IOC_READ, 'P', 1, DEVICEID_LENGTH), deviceid);
		if (iostatus == 0) {
			if (strstr(deviceid + 2, "EPSON")) {
				if (strstr(deviceid + 2, "ESCPR")==NULL) {
					result = 1; /* found */
				}
			}
		}

		close(fd);
	}

	return result;
}


static int 
get_escp_printer_devpath (char * path, int sizePath)
{
	struct dirent * lp = NULL;
	DIR * devpath = opendir("/dev/usb");
	int error = 1;

	if (devpath) {
		while ((lp = readdir(devpath)) != NULL) {
			if (lp->d_name[0] == '.') {
				continue;
			}

			if (strncmp(lp->d_name, "lp", 2) != 0) {
				continue;
			}

			if (is_escp_printer(lp->d_name) == 1) { /* ESCP printer found */
				memset(path, 0x00, sizePath);
				snprintf(path, sizePath, "/dev/usb/%s", lp->d_name);
				error = 0;
				break;

			}
			
			
		}

		closedir(devpath);
	}

	return error;
}

int
parameter_update (P_CBTD_INFO p_info)
{
	return get_escp_printer_devpath (p_info->devprt_path, sizeof(p_info->devprt_path));
}


int
parameter_setup (P_CBTD_INFO p_info)
{
    char line [1024];    
    char name[255];
    char *def_name, *value = NULL;
    int i;
    FILE* file;
    char rc_file[] = PIPS_EKPD_RC_PATH;

    file = fopen (rc_file, "rt");
    if (file == NULL)
      return 1;

    /* Get default printer name */
    while (!feof(file))
      {
	fgets (line, 1024, file);
	if (!strstr (line, "defaultprinter"))
	  continue;	
	def_name = strchr (line, '>') + 1;
	i = strchr (def_name, '<') - def_name;
	def_name[i] = '\0';
	/* store the default name */
	strcpy (name, def_name); 
	break;	
      } 

    if (name == NULL) 
      return 1; 		/* not contain default printer section */
  
    while (!feof(file))
      {
	fgets (line, 1024, file);

	/* Seeking to the section of the printer */
	if (!strstr (line, name) || (!strstr (line, "<printer name=")))
	  continue;
	/* get information */
	while (1)
	  {	    
	    fgets (line, 1024, file );    

	    if (strstr (line , "</printer>"))
	      break;

	    value = strchr (line, '>') + 1;
	    i = strchr (value, '<') - value;
	    value[i] = '\0';	    
	    
	    if (strstr (line ,"<aliasname>" ))
	      {
		strcpy (p_info->printer_name, value);
		continue;
	      }

	    if (strstr (line , "<devicepath>"))
	      {
		strcpy (p_info->devprt_path, value);
		continue;
	      }

	    if (strstr (line , "<dummypath>"))
	      {
		strcpy (p_info->infifo_path, value);
		continue;
	      }

	    if (strstr (line , "<serverport>"))
	      {
		p_info->comsock_port = (int) atoi ((char *)value);
		continue;
	      }		   
	  }
	break;
      }


    if (*(p_info->printer_name) == 0
            || *(p_info->devprt_path) == 0
            || *(p_info->infifo_path) == 0
            || p_info->comsock_port < 0
            || p_info->comsock_port > UINT_MAX)
        return 1;    

    return 0; 
}

