/*
 * Photo Image Print System
 * Copyright (C) 2000-2005 EPSON AVASYS Corporation.
 * Copyright (C) SEIKO EPSON CORPORATION 2000-2005.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, EPSON AVASYS Corporation gives permission to
 * link the code of this program with libraries which are covered by
 * the EPSON AVASYS Public License and distribute their linked
 * combinations.  You must obey the GNU General Public License in all
 * respects for all of the code used other than the libraries which
 * are covered by EPSON AVASYS Public License.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifndef __PIPS_CAP_TYPE_H__
#define __PIPS_CAP_TYPE_H__

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

#include "pipscap-def.h"

    typedef struct _POINT
    {
        long x;
        long y;
    }
    POINT, *LP_POINT;

    typedef struct _PAPER_SIZE_STRUCT
    {
        POINT size;
        POINT area;
        POINT def_area;
        POINT margin;
        POINT def_margin;
    }
    PS_STRUCT, *LP_PS_STRUCT;

    /* Cap context list */
    typedef struct CAP_CONTEXT_NODE
    {
        struct CAP_CONTEXT_NODE *next;
        int capid;
        char *option;
    }
    CapContextNode, *CapContextList;

    /* Database structure */
    /* Cap list */
    typedef struct ELEMENT_NODE
    {
        struct ELEMENT_NODE *next;
        char *option;
        char *value;
    }
    ElementNode, *ElementList;

    typedef struct CAPABILITY_NODE
    {
        struct CAPABILITY_NODE *next;
        int capid;
        char *param;
        ElementList list;
    }
    CapNode, *CapList;

    typedef struct PAPER_SIZE_NODE
    {
        struct PAPER_SIZE_NODE *next;
        char *ptid;
        long width;
        long height;
    }
    PaperSizeNode, *PaperSizeList;

    typedef struct CONSTRAINT_PAPER_SIZE_NODE
    {
        struct CONSTRAINT_PAPER_SIZE_NODE *next;
        char *ptid;
        long pa_width;
        long pa_length;
        long right_margin;
        long left_margin;
        long top_margin;
        long bottom_margin;
    }
    ConsPaperSizeNode, *ConsPaperSizeList;

    /* Constraint Paper Source value */
    typedef struct CONSTRAINT_PAPER_SOURCE
    {
        struct CONSTRAINT_PAPER_SOURCE *next;
        char *psid;
        ConsPaperSizeList list;
    }
    ConsPaperSourceNode, *ConsPaperSourceList;

    /* Print mode value */
    typedef struct PRINT_MODE_NODE
    {
        struct  PRINT_MODE_NODE *next;
        int modeid;
        char *mode_name;
        CapContextList list;
    }
    PrintModeNode, *PrintModeList;

    /* Database */
    typedef struct CAP_DATABASE
    {
        CapList cap_list;		   /* Option list */
        CapList common_list;	   /* Common list */
        PaperSizeList papersize_list;  /* Paper size list */
        ConsPaperSourceList cons_list; /* Constraint paper size list */
        PrintModeList mode_color_list; /* Print mode list */
        PrintModeList mode_mono_list;
        PrintModeList mode_abwp_list;
        CapContextList name_def_list;  /* Printer name, Model name, Filter name */
        CapContextList other_list;     /* Inkset, Inkbox type, Printer type */
    }
    CapDatabase, *CapDatabasePtr;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif //__PIPS_CAP_TYPE_H__
