/*
 * Photo Image Print System
 * Copyright (C) 2000-2005 EPSON AVASYS Corporation.
 * Copyright (C) SEIKO EPSON CORPORATION 2000-2005.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, EPSON AVASYS Corporation gives permission to
 * link the code of this program with libraries which are covered by
 * the EPSON AVASYS Public License and distribute their linked
 * combinations.  You must obey the GNU General Public License in all
 * respects for all of the code used other than the libraries which
 * are covered by EPSON AVASYS Public License.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <ltdl.h>
#include "pipserr.h"
#include "pips-filter.h"
#include "pngfilter.h"
#include "rawfilter.h"

/***************************************************
 *                    main
 ***************************************************/
static int GetFilterType (int, char**);

int main( int argc, char **argv )
{
    int error = 0;

    /* Start  */
    bindtextdomain (PACKAGE, LOCALE_PATH);
    textdomain (PACKAGE);

    error = lt_dlinit();
    if (error != 0)
    {
        FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror());
        goto exit;
    }

    /* Processing parameters */
    switch (GetFilterType (argc, argv))
    {
    case 0:
        error = 1;
        break;

    case 1: /* Call png filter from pips-tool, command line or lpr-filter */
        error = PngFilter (argc, argv);
        break;

    case 2: /* Cal raw filter from cups-filter */
        error = RawFilter (argc, argv);
        break;
    }

    lt_dlexit();
    /* End */
exit:
    return error ? 1 : 0;
}

/* Determine the PIPS filter caller */
static int
GetFilterType (int argc, char **argv)
{
    int index;
    char *filter;
    int filterType = 0;

    if ((index = GetGlobalOpt (argc, argv, "f")) > 0)
    {
        filter = argv[index];
        if (!strcmp (filter, "png"))
            filterType = 1;
        else if (!strcmp (filter, "raw"))
            filterType = 2;
        else
            FILTER_ERROR_HANDLER (INVALID_FILTER_ERROR, filter);
    }

    return filterType;
}

/* For getting some global options */
int
GetGlobalOpt (int nargc, char **argv, char *opt)
{
    int argc;

    for (argc = 1; argc < nargc; argc++)
    {
        if ((argv[argc][0] == '-') && (!strcmp (argv[argc] + 1, opt)))
        {
            if ((++argc < nargc) && ((argv[argc][0] != '-') || (isdigit (argv[argc][1]))))
                return argc;
            else
            {
                opt = argv[argc-1];
                FILTER_ERROR_HANDLER (OPTION_LESS_ERROR, opt);
                return -1;
            }
        }
    }
    return 0;
}


