/*
 * Photo Image Print System
 * Copyright (C) 2000-2005 EPSON AVASYS Corporation.
 * Copyright (C) SEIKO EPSON CORPORATION 2000-2005.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, EPSON AVASYS Corporation gives permission to
 * link the code of this program with libraries which are covered by
 * the EPSON AVASYS Public License and distribute their linked
 * combinations.  You must obey the GNU General Public License in all
 * respects for all of the code used other than the libraries which
 * are covered by EPSON AVASYS Public License.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ltdl.h>
#include "pips-filter.h"
#include "pfimg.h"
#include "pipserr.h"
#include "pipsmem.h"
#include "pips-caplib.h"
#include "setup.h"
#include "pngfilter.h"

static BOOL png_change_prtinfo (lt_dlhandle, CapHandle, LP_MKC_STRUCT, CapContextList*);
static int graphics_converter_version2 (lt_dlhandle, CapHandle, CapContextList, char*, int, char*, int);

int
PngFilter(int argc, char **argv)
{
    char *model_name = NULL, *alias_name = NULL;
    char *rctemp_path = NULL;

    /* for attribute bitmap */
    char *attrbmp_path = NULL;
    int attrbmp = 0;

    char cap_path[] = LIB_CAP_PATH;
    int ret_code, i, called_type = 0;
    char err_data[MAX_SIZE];
    CapContextList p_caplist = NULL;
    lt_dlhandle lib_handle;
    CapHandle cap_handle;
    LOADCAPCTX_FUNC LoadCap;
    OPENCAP_FUNC OpenCap;
    SETCAPLIST_FUNC SetCapList;
    FIXCAPCTX_FUNC FixCap;
    REFCAPCTX_FUNC ReferCap;

    /* Initialize */
    memset (err_data, '\0', MAX_SIZE);
    /* Load capability library */
    lib_handle = lt_dlopen (cap_path);
    if (!lib_handle)
    {
        FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror());
        return 1;
    }

    if ((i = GetGlobalOpt (argc, argv, "p")))
    {
        if (i == -1)
            return 1;
        model_name = argv[i];
    }

    if ((i = GetGlobalOpt (argc, argv, "r")))
    {
        if (i == -1)
            return 1;
        rctemp_path = argv[i];
        if (CheckExistFile (rctemp_path))
            return 1;
        called_type = 1; /* Called from pips-tool */
    }

    /* attribute bitmap */
    if ((i = GetGlobalOpt (argc, argv, "at")))
    {
        if (i == -1)
            return 1;
        attrbmp_path = argv[i];
        if (CheckExistFile (attrbmp_path))
            return 1;
        attrbmp = 1;
    }

    if (model_name == NULL)
    {
        if (!(model_name = GetDefaultPrinter(1)))
        {
            FILTER_ERROR_HANDLER (DEF_PRINTER_NOT_FOUND, NULL);
            char **model_list = NULL;
            model_list = GetModelListName (PNG_TYPE);
            if (*model_list)
            {
                model_name = *model_list;
            }

            else
            {
                FILTER_ERROR_HANDLER (PRINTER_NOT_FOUND, NULL);
                return 1;
            }
        }
    }

    alias_name = GetAliasName(model_name);
    if (alias_name == NULL) {
	FILTER_ERROR_HANDLER (PRINTER_NOT_FOUND, NULL);
	return 1;
    }

    memset (err_data, '\0', MAX_SIZE);
    OpenCap = (OPENCAP_FUNC) lt_dlsym (lib_handle, "OpenCap");
    if (OpenCap == NULL)
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror());
        return 1;
    }
    if ((ret_code = (*OpenCap) (&cap_handle, model_name, PNG_TYPE, err_data)))
    {
        if (ret_code != PIPSRC_NOT_FOUND)
        {
            lt_dlclose (lib_handle);
            FILTER_ERROR_HANDLER (ret_code, err_data);
            return 1;
        }
        ret_code = 0;
    }

    /* Check & Set option list for SubContext */
    SetCapList = (SETCAPLIST_FUNC)lt_dlsym (lib_handle, "SetCapList");
    if (SetCapList == NULL )
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror());
        return 1;
    }

    if ((ret_code = (*SetCapList) (cap_handle, argv, argc, err_data))) /* Update ContextList */
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (ret_code, err_data);
        return 1;
    }

    /* Options are OK! */
    /* Update context from rcTemp file that called by pips-tool if have*/
    if (rctemp_path)
    {
        LoadCap = (LOADCAPCTX_FUNC) lt_dlsym (lib_handle, "LoadCapContext");

        if (LoadCap == NULL)
        {
            lt_dlclose (lib_handle);
            FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror());
            return 1;
        }

        if ((ret_code = (*LoadCap) (cap_handle, rctemp_path)))
        {
            lt_dlclose (lib_handle);
            FILTER_ERROR_HANDLER (ret_code, rctemp_path);
            return 1;
        }
    }

    /* Have to FixCapContext for checking the input params */
    FixCap = (FIXCAPCTX_FUNC)lt_dlsym (lib_handle, "FixCapContext");
    if (FixCap == NULL )
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror());
        return 1;
    }
    if ((ret_code = (*FixCap) (cap_handle, FALSE)))
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (ret_code, cap_path);
        return 1;
    }

    /* Get CapContextList */
    ReferCap = (REFCAPCTX_FUNC)lt_dlsym (lib_handle, "ReferCapContext");
    if (ReferCap == NULL )
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror());
        return 1;
    }
    if ((ret_code = (*ReferCap) (cap_handle, &p_caplist)))
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (ret_code, cap_path);
        return 1;
    }

    ret_code = graphics_converter_version2 (lib_handle, cap_handle, p_caplist, alias_name, called_type, attrbmp_path, attrbmp);
    if (lib_handle)
    {
        if (lt_dlclose (lib_handle) == -1)
        {
            FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror ());
            return 1;
        }
    }

    return ret_code;
}

static int
graphics_converter_version2 (lt_dlhandle libcap_handle, CapHandle cap_handle, CapContextList p_caplist, char *alias_name, int called_type, char* attrbmp_path, int attrbmp)
{
    LP_MKC_STRUCT lp_mkcode = pips_malloc (MKC_STRUCT, 1);
    char *input_file, *output_file, *libprt_path;
    char *input_files[] = {NULL, NULL};
    char *clear_ink, *correction_color;

    /* Printer lib path */
    if (!(libprt_path = GetLibPrtPath (GetCapValue (p_caplist, P_MODEL_NAME))))
        return 1;
    lp_mkcode->atbmp_path = NULL;
    lp_mkcode->libprt_path = libprt_path;
    /* Input file path */
    if (!(input_file = GetCapValue (p_caplist, P_INPUT_FILE)))
        input_file = strdup (STD_IN_NAME);
    else
    {   /* Check input data file */
        if (CheckExistFile (input_file))
        {
            FILTER_ERROR_HANDLER (FILE_ACCESS_ERROR, input_file);
            return 1;
        }
    }
    if (strcmp (input_file, "STDIN"))
    {
        input_files[0] = input_file;
        lp_mkcode->input_files = input_files;
    }
    else
        lp_mkcode->input_files = NULL;
    /* Output file path */
    output_file = GetCapValue (p_caplist, P_OUTPUT_FILE);
    lp_mkcode->output_file = output_file;

    if (!output_file && (called_type == 1)) /* Called from pips-tool */
            lp_mkcode->output_file = ".";

    /* Attribute bitmap file path*/ 
    /* Not get the path from the xml file but from the user input */
    if (!attrbmp)
      {
	/* Validate the attribute bitmap path */
	clear_ink = GetCapValue(p_caplist, P_CLEAR_INK);
	if (clear_ink != NULL)
	  {
	    if (!strcmp(clear_ink, "AUTO"))	  
	      {
		fprintf (stderr, "Missing attribute.\nSetting Gloss Optimizer to \"ON(FULL)\".\n");	    
		UpdateContextList(p_caplist, P_CLEAR_INK, "FULL");
	      }	
	  }
	correction_color = GetCapValue(p_caplist, P_COLOR_MODE);
	if (correction_color != NULL)
	  {
	    if (!strcmp(correction_color, "AUTO"))
	      {
		fprintf (stderr, "Missing attribute.\nSetting color correction to \"Photo\".\n");
		UpdateContextList(p_caplist, P_COLOR_MODE, "PHOTO");
	      }
	  }
      }
    else	  
	lp_mkcode->atbmp_path = attrbmp_path;/* GetCapValue (p_caplist, P_ATBMP_FILE); */
      
    /* Alias name */
    lp_mkcode->alias_name = alias_name;
    /* Get and convert some size information */
    if (png_change_prtinfo (libcap_handle, cap_handle, lp_mkcode, &p_caplist))
        return 1;
    lp_mkcode->p_caplist = p_caplist;

    return (int)make_code_process (lp_mkcode);
}

static BOOL
png_change_prtinfo (lt_dlhandle lib_handle, CapHandle cap_handle, LP_MKC_STRUCT lp_mkcode, CapContextList *p_caplist)
{
    char cap_path[] = LIB_CAP_PATH, *value;
    int resol, ret_code;
    long area_y = 0, margin_x, margin_y;
    LP_PS_STRUCT paper_conf, paper_src;
    GETPAPERRSC_FUNC GetPaperRsc;

    /* Initialize */
    paper_src = pips_malloc (PS_STRUCT, 1);
    paper_conf = pips_malloc (PS_STRUCT, 1);
    /* Get Paper Size correspondence with Paper Source */
    GetPaperRsc = (GETPAPERRSC_FUNC)lt_dlsym (lib_handle, "GetPaperRsc");
    if (GetPaperRsc == NULL)
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (LIB_ERROR, (char*)lt_dlerror());
        return 1;
    }

    if ((ret_code = (*GetPaperRsc) (cap_handle, GetCapValue (*p_caplist, P_PAPER_SOURCE),
                                    GetCapValue (*p_caplist, P_MEDIA_SIZE), paper_src)))
    {
        lt_dlclose (lib_handle);
        FILTER_ERROR_HANDLER (ret_code, cap_path);
        return 1;
    }
    if (!( resol = atoi(GetCapValue (*p_caplist, P_RESOLUTION))))
        return 1;
    /* Support Wide Print */
    if ((value = GetCapValue (*p_caplist, P_WIDE_PRINT)))
    {
        if (strcmp ( value, "ON") == 0)
	    area_y = paper_src->size.y - (paper_src->margin.y + 42);
        else
            area_y = paper_src->area.y;
    }

    paper_conf->size.x       = paper_src->size.x * resol / 360;
    paper_conf->size.y       = paper_src->size.y * resol / 360;
    paper_conf->area.x       = paper_src->area.x * resol / 360;
    paper_conf->area.y       = paper_src->area.y * resol / 360;
    paper_conf->def_area.x   = paper_conf->area.x;
    paper_conf->def_area.y   = area_y * resol / 360;
    paper_conf->def_margin.x = paper_src->margin.x * resol / 360;
    paper_conf->def_margin.y = paper_src->margin.y * resol / 360;

    /* Not for fit to page */
    if (strcmp (GetCapValue (*p_caplist, P_FIT_PAGE), "ON"))
    {
        margin_x = MM_TO_DOT( atol (GetCapValue (*p_caplist, P_MARGIN_X)));
        if (margin_x >= (paper_src->margin.x + paper_src->area.x))
            paper_conf->margin.x = (paper_src->margin.x + paper_src->area.x) * resol / 360 - 1;
        else
            paper_conf->margin.x = margin_x * resol / 360;

        margin_y = MM_TO_DOT( atol (GetCapValue (*p_caplist, P_MARGIN_Y)));
        if (margin_y >= (paper_src->margin.y + area_y))
            paper_conf->margin.y = (paper_src->margin.y + area_y) * resol / 360 - 1;
        else
            paper_conf->margin.y = margin_y * resol / 360;
    }
    else
    {
        paper_conf->margin.x = paper_src->margin.x * resol / 360;
        paper_conf->margin.y = paper_src->margin.y * resol / 360;
    }

    value = GetCapValue (*p_caplist, P_PAPER_SOURCE);
    if (value == NULL)
    {
        FILTER_ERROR_HANDLER (MEMORY_ERROR, NULL);
        return 1;
    }
    if (strcmp (value, "AUTO") && strcmp (value, "ROLL") &&
        strcmp (value, "MANUAL") && strcmp (value, "RMANUAL")
        && strcmp (value, "REAR"))
    {
        paper_conf->margin.x = 0;
        if (!strcmp (value, "T4AUTO") || !strcmp (value, "T4REAR"))
    	paper_conf->margin.y = 0;
    }

    if ((value = GetCapValue (*p_caplist, P_AUTO_CUT)))
    {
        if (!strcmp (value, "2")) 	/* top (42) + bottom (42) */
            paper_conf->area.y  += (84 * resol) / 360;
    }

    paper_conf->area.x = (paper_conf->margin.x < paper_conf->def_margin.x)
                         ? paper_conf->area.x : paper_conf->area.x - paper_conf->margin.x + paper_conf->def_margin.x;
    paper_conf->area.y = (paper_conf->margin.y < paper_conf->def_margin.y)
                         ? paper_conf->area.y : paper_conf->area.y - paper_conf->margin.y + paper_conf->def_margin.y;

    if (AddPaperRsc (p_caplist, paper_conf, TRUE))
        return 1;

    pips_free (paper_conf);
    pips_free (paper_src);

    return NO_ERROR;
}
