/*
 * Photo Image Print System
 * Copyright (C) 2000-2005 EPSON AVASYS Corporation.
 * Copyright (C) SEIKO EPSON CORPORATION 2000-2005.
 *
 *  This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * As a special exception, EPSON AVASYS Corporation gives permission to
 * link the code of this program with libraries which are covered by
 * the EPSON AVASYS Public License and distribute their linked
 * combinations.  You must obey the GNU General Public License in all
 * respects for all of the code used other than the libraries which
 * are covered by EPSON AVASYS Public License.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "pipserr.h"

#define EKPD_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define EKPD_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

GtkWidget*
create_ekpdtool (void)
{
  GtkWidget *ekpdtool;
  AtkObject *atko;
  GtkWidget *vbox1;
  GtkWidget *table1;
  GtkWidget *label2;
  GtkWidget *label1;
  GtkWidget *label3;
  GtkWidget *label4;
  GtkWidget *entryAliasName;
  GtkWidget *checkbutton1;
  GtkWidget *label5;
  GtkWidget *optionModel;
  GtkWidget *combo_entry1;
  GtkWidget *combo_list1;
  GtkWidget *combo_list2;
  GtkWidget *optionDevice;
  GtkWidget *combo_entry2;
  GtkWidget *optionPort;
  GtkWidget *combo_entry3;
  GtkWidget *hbox1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  char *temp_cupsid_file;
  int fd = 0;
  char com_buf[256];
  struct stat file_status;


  ekpdtool = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_container_set_border_width (GTK_CONTAINER (ekpdtool), 5);
  gtk_window_set_title (GTK_WINDOW (ekpdtool), _("Connection"));
  gtk_window_set_position (GTK_WINDOW (ekpdtool), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (ekpdtool), 330, 180);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (ekpdtool), vbox1);

  table1 = gtk_table_new (5, 2, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);

  label2 = gtk_label_new (_("Connection Method"));
  gtk_widget_show (label2);
  gtk_table_attach (GTK_TABLE (table1), label2, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

  label1 = gtk_label_new (_("Printer Name"));
  gtk_widget_show (label1);
  gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  label3 = gtk_label_new (_("Device Path"));
  gtk_widget_show (label3);
  gtk_table_attach (GTK_TABLE (table1), label3, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

  label4 = gtk_label_new (_("Alias Name"));
  gtk_widget_show (label4);
  gtk_table_attach (GTK_TABLE (table1), label4, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

  entryAliasName = gtk_entry_new ();
  gtk_widget_show (entryAliasName);
  gtk_table_attach (GTK_TABLE (table1), entryAliasName, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  temp_cupsid_file = strdup ("/var/tmp/pips-cupsid-temp.XXXXXX");
  fd = mkstemp (temp_cupsid_file);
  chmod (temp_cupsid_file, 0666);
  if (fd == -1)
        FILTER_ERROR_HANDLER (FILE_ACCESS_ERROR, NULL);
  sprintf (com_buf, "%s%s","/sbin/pidof cupsd > ",temp_cupsid_file);
  system(com_buf);

  if (!stat(temp_cupsid_file,&file_status) && (file_status.st_size> 1))
        gtk_widget_set_sensitive(entryAliasName,0);
  sprintf (com_buf, "%s%s%s","rm -f"," ",temp_cupsid_file);
  system(com_buf);

  checkbutton1 = gtk_check_button_new_with_mnemonic (_("Default Printer"));
  gtk_widget_show (checkbutton1);
  gtk_table_attach (GTK_TABLE (table1), checkbutton1, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1), FALSE);

  label5 = gtk_label_new ("");
  gtk_widget_show (label5);
  gtk_table_attach (GTK_TABLE (table1), label5, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

  optionModel = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (optionModel)->popwin),
                     "EkpdKey", optionModel);
  gtk_widget_show (optionModel);
  gtk_table_attach (GTK_TABLE (table1), optionModel, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_combo_set_value_in_list (GTK_COMBO (optionModel), TRUE, TRUE);

  combo_entry1 = GTK_COMBO (optionModel)->entry;
  gtk_widget_show (combo_entry1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);
  //gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (combo_entry1), 0);

  combo_list1 = GTK_COMBO (optionModel)->list;
  gtk_widget_show (combo_list1);
  optionDevice = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (optionDevice)->popwin),
                     "EkpdKey", optionDevice);
  gtk_widget_show (optionDevice);
  gtk_table_attach (GTK_TABLE (table1), optionDevice, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_list2 = GTK_COMBO (optionDevice)->list;
  gtk_widget_show (combo_list2);

  combo_entry2 = GTK_COMBO (optionDevice)->entry;
  gtk_widget_show (combo_entry2);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry2), FALSE);

  optionPort = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (optionPort)->popwin),
                     "EkpdKey", optionPort);
  gtk_widget_show (optionPort);
  gtk_table_attach (GTK_TABLE (table1), optionPort, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  combo_entry3 = GTK_COMBO (optionPort)->entry;
  gtk_widget_show (combo_entry3);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry3), FALSE);

  hbox1 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, GTK_BUTTONBOX_END);

  button1 = gtk_button_new_with_mnemonic (_("Test"));
  //gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox1), button1, TRUE, TRUE, 5);

  button2 = gtk_button_new_with_mnemonic (_("OK"));
  gtk_widget_show (button2);
  gtk_box_pack_start (GTK_BOX (hbox1), button2, TRUE, TRUE, 5);

  button3 = gtk_button_new_with_mnemonic (_("Cancel"));
  gtk_widget_show (button3);
  gtk_box_pack_start (GTK_BOX (hbox1), button3, TRUE, TRUE, 5);

  g_signal_connect ((gpointer) ekpdtool, "destroy",
                    G_CALLBACK (gtk_main_quit),
                    NULL);
  g_signal_connect_swapped ((gpointer) button1, "clicked",
                            G_CALLBACK (on_test_clicked),
                            G_OBJECT (ekpdtool));
  g_signal_connect_swapped ((gpointer) button2, "clicked",
                            G_CALLBACK (on_ok_clicked),
                            G_OBJECT (ekpdtool));
  g_signal_connect_swapped ((gpointer) button3, "clicked",
                            G_CALLBACK (gtk_main_quit),
                            G_OBJECT (ekpdtool));
  g_signal_connect(G_OBJECT(combo_list1),"select_child",
					   G_CALLBACK(on_combo_list1_select_child),NULL);
  g_signal_connect(G_OBJECT(combo_list2),"select_child",
					   G_CALLBACK(on_combo_list2_select_child),NULL);
					   
  atko = gtk_widget_get_accessible (ekpdtool);
  atk_object_set_name (atko, "ekpd-tool");


  /* Store pointers to all widgets, for use by lookup_widget(). */
  EKPD_HOOKUP_OBJECT_NO_REF (ekpdtool, ekpdtool, "ekpdtool");
  EKPD_HOOKUP_OBJECT (ekpdtool, vbox1, "vbox1");
  EKPD_HOOKUP_OBJECT (ekpdtool, table1, "table1");
  EKPD_HOOKUP_OBJECT (ekpdtool, label2, "label2");
  EKPD_HOOKUP_OBJECT (ekpdtool, label1, "label1");
  EKPD_HOOKUP_OBJECT (ekpdtool, label3, "labelPort");
  EKPD_HOOKUP_OBJECT (ekpdtool, label4, "label4");
  EKPD_HOOKUP_OBJECT (ekpdtool, entryAliasName, "entryAliasName");
  EKPD_HOOKUP_OBJECT (ekpdtool, checkbutton1, "checkbutton1");
  EKPD_HOOKUP_OBJECT (ekpdtool, label5, "label5");
  EKPD_HOOKUP_OBJECT (ekpdtool, optionModel, "optionModel");
  EKPD_HOOKUP_OBJECT (ekpdtool, combo_entry1, "combo_entry1");
  EKPD_HOOKUP_OBJECT (ekpdtool, optionDevice, "optionDevice");
  EKPD_HOOKUP_OBJECT (ekpdtool, combo_entry2, "combo_entry2");
  EKPD_HOOKUP_OBJECT (ekpdtool, optionPort, "optionPort");
  EKPD_HOOKUP_OBJECT (ekpdtool, combo_entry3, "combo_entry3");
  EKPD_HOOKUP_OBJECT (ekpdtool, hbox1, "hbox1");
  EKPD_HOOKUP_OBJECT (ekpdtool, button1, "button1");
  EKPD_HOOKUP_OBJECT (ekpdtool, button2, "buttonOk");
  EKPD_HOOKUP_OBJECT (ekpdtool, button3, "button3");
  EKPD_HOOKUP_OBJECT (ekpdtool, combo_entry1, "combo_entry1");
  EKPD_HOOKUP_OBJECT (ekpdtool, checkbutton1, "checkbutton1");
  EKPD_HOOKUP_OBJECT (ekpdtool, combo_list1, "combo_list1");
  EKPD_HOOKUP_OBJECT (ekpdtool, combo_entry1, "combo_list2");
  return ekpdtool;
}

GtkWidget*
create_msg_no_authority (void)
{
    GtkWidget *msg_no_authority;
    GtkWidget *dialog_vbox1;
    GtkWidget *msg_label;
    GtkWidget *dialog_action_area1;
    GtkWidget *msg_ok_button;

    msg_no_authority = gtk_dialog_new ();
    g_object_set_data (G_OBJECT (msg_no_authority), "msg_no_authority", msg_no_authority);
    
    gtk_window_set_title (GTK_WINDOW (msg_no_authority), _("PIPS Warning"));
    gtk_window_set_modal (GTK_WINDOW (msg_no_authority), TRUE);
    gtk_window_set_policy (GTK_WINDOW (msg_no_authority), FALSE, FALSE, FALSE);

    dialog_vbox1 = GTK_DIALOG (msg_no_authority)->vbox;
    g_object_set_data (G_OBJECT (msg_no_authority), "dialog_vbox1", dialog_vbox1);
    gtk_widget_show (dialog_vbox1);

    msg_label = gtk_label_new (_("Please must run by root authority."));
    gtk_widget_ref (msg_label);
    g_object_set_data_full (G_OBJECT (msg_no_authority), "msg_label", msg_label,
                              (GDestroyNotify) gtk_widget_unref);
    gtk_widget_show (msg_label);
    gtk_box_pack_start (GTK_BOX (dialog_vbox1), msg_label, FALSE, FALSE, 0);
    gtk_label_set_line_wrap (GTK_LABEL (msg_label), TRUE);
    gtk_misc_set_padding (GTK_MISC (msg_label), 20, 20);

    dialog_action_area1 = GTK_DIALOG (msg_no_authority)->action_area;
    g_object_set_data (G_OBJECT (msg_no_authority), "dialog_action_area1", dialog_action_area1);
    gtk_widget_show (dialog_action_area1);
    gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

    msg_ok_button = gtk_button_new_with_label (_("OK"));
    gtk_widget_ref (msg_ok_button);
    g_object_set_data_full (G_OBJECT (msg_no_authority), "msg_ok_button", msg_ok_button,
                              (GDestroyNotify) gtk_widget_unref);
    gtk_widget_show (msg_ok_button);
    gtk_box_pack_start (GTK_BOX (dialog_action_area1), msg_ok_button, FALSE, TRUE, 0);

    g_signal_connect (G_OBJECT (msg_ok_button), "clicked",
                        G_CALLBACK (on_msg_ok_button_clicked),
                        NULL);
    g_signal_connect (G_OBJECT (msg_no_authority), "delete_event", G_CALLBACK (gtk_main_quit), NULL);
    g_signal_connect (G_OBJECT (msg_no_authority), "destroy", G_CALLBACK(gtk_main_quit), NULL);

    return msg_no_authority;
}
